#ifndef _ISL_INCLUDE_ISL_STDINT_H
#define _ISL_INCLUDE_ISL_STDINT_H 1
#ifndef _GENERATED_STDINT_H
#define _GENERATED_STDINT_H "isl 0.15"
/* generated using gnu compiler aarch64-conda-linux-gnu-cc (crosstool-NG 1.22.0.1750_510dbc6_dirty) 10.2.0 */
#define _STDINT_HAVE_STDINT_H 1
#include <stdint.h>
#endif
#endif
